# This file is part of monday-client.
#
# Copyright (C) 2024 Leet Cyber Security <https://leetcybersecurity.com/>
#
# monday-client is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# monday-client is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with monday-client. If not, see <https://www.gnu.org/licenses/>.

# pylint: disable=redefined-outer-name

from unittest.mock import AsyncMock, MagicMock

import pytest

from monday.client import MondayClient
from monday.services.items import Items


@pytest.fixture(scope="module")
def mock_client():
    return MagicMock(spec=MondayClient)


@pytest.fixture(scope="module")
def items_instance(mock_client):
    return Items(mock_client)


@pytest.mark.asyncio
async def test_query(items_instance):
    mock_responses = [
        {'data': {'items': [{'id': 1, 'name': 'Item 1'}, {'id': 2, 'name': 'Item 2'}]}},
        {'data': {'items': []}}
    ]

    items_instance.client.post_request = AsyncMock(side_effect=mock_responses)
    result = await items_instance.query(item_ids=[1, 2])

    assert result == [{'id': 1, 'name': 'Item 1'}, {'id': 2, 'name': 'Item 2'}]
    assert items_instance.client.post_request.await_count == 2


@pytest.mark.asyncio
async def test_create(items_instance):
    mock_response = {
        'data': {
            'create_item': {'id': 1, 'name': 'New Item'}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.create(board_id=1, item_name="New Item")

    assert result == {'id': 1, 'name': 'New Item'}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_duplicate(items_instance):
    mock_response = {
        'data': {
            'duplicate_item': {'id': 2, 'name': 'Item 1 (copy)'}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.duplicate(item_id=1, board_id=1)

    assert result == {'id': 2, 'name': 'Item 1 (copy)'}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_move_to_group(items_instance):
    mock_response = {
        'data': {
            'move_item_to_group': {'id': 1, 'group': {'id': 'new_group'}}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.move_to_group(item_id=1, group_id='new_group')

    assert result == {'id': 1, 'group': {'id': 'new_group'}}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_move_to_board(items_instance):
    mock_response = {
        'data': {
            'move_item_to_board': {'id': 1, 'board': {'id': 2}}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.move_to_board(
        item_id=1,
        board_id=2,
        group_id='new_group',
        columns_mapping=None,
        subitems_columns_mapping=None
    )

    assert result == {'id': 1, 'board': {'id': 2}}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_archive(items_instance):
    mock_response = {
        'data': {
            'archive_item': {'id': 1, 'state': 'archived'}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.archive(item_id=1)

    assert result == {'id': 1, 'state': 'archived'}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_delete(items_instance):
    mock_response = {
        'data': {
            'delete_item': {'id': 1, 'state': 'deleted'}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.delete(item_id=1)

    assert result == {'id': 1, 'state': 'deleted'}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_clear_updates(items_instance):
    mock_response = {
        'data': {
            'clear_item_updates': {'id': 1, 'updates_cleared': True}
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.clear_updates(item_id=1)

    assert result == {'id': 1, 'updates_cleared': True}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_items_page_by_column_values(items_instance):
    mock_responses = [{
        'data': {
            'items_page_by_column_values': {
                'cursor': None,
                'items': [
                    {'id': 1, 'name': 'Item 1'},
                    {'id': 2, 'name': 'Item 2'}
                ]
            }
        }
    }]

    items_instance.client.post_request = AsyncMock(side_effect=mock_responses)
    result = await items_instance.items_page_by_column_values(
        board_id=1,
        columns=[{'column_id': 'status', 'column_values': ['Done']}],
        limit=2
    )

    assert result == {'items': [{'id': 1, 'name': 'Item 1'}, {'id': 2, 'name': 'Item 2'}]}
    items_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_items_page(items_instance):
    mock_response = {
        'data': {
            'boards': [
                {
                    'id': 1,
                    'items_page': {
                        'cursor': None,
                        'items': [
                            {'id': 1, 'name': 'Item 1'},
                            {'id': 2, 'name': 'Item 2'}
                        ]
                    }
                }
            ]
        }
    }

    items_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await items_instance.items_page(board_ids=1, limit=2)

    assert result == [{'board_id': 1, 'items': [{'id': 1, 'name': 'Item 1'}, {'id': 2, 'name': 'Item 2'}]}]
    items_instance.client.post_request.assert_awaited_once()



# This file is part of monday-client.
#
# Copyright (C) 2024 Leet Cyber Security <https://leetcybersecurity.com/>
#
# monday-client is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# monday-client is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with monday-client. If not, see <https://www.gnu.org/licenses/>.

"""Custom exceptions for the Monday module."""

from typing import Any, Dict, Optional


class ComplexityLimitExceeded(Exception):
    """
    Exception raised when the complexity limit is exceeded.

    Attributes:
        message (str): Explanation of the error.
        reset_in (int): Time in seconds until the complexity limit is reset.
    """

    def __init__(self, message: str, reset_in: int):
        super().__init__(message)
        self.reset_in = reset_in


class MondayAPIError(Exception):
    """
    Exception raised when an error occurs during API communication with Monday.com.

    Attributes:
        message (str): Explanation of the error.
        json_data (Dict[str, Any], optional): JSON data returned by the API, if available.
    """

    def __init__(self, message, json_data: Optional[Dict[str, Any]] = None):
        super().__init__(message)
        self.json: Optional[Dict[str, Any]] = json_data


class MutationLimitExceeded(Exception):
    """
    Exception raised when the mutation per minute limit is exceeded.

    Attributes:
        message (str): Explanation of the error.
        reset_in (int): Time in seconds until the rate limit is reset.
    """

    def __init__(self, message: str, reset_in: int):
        super().__init__(message)
        self.reset_in = reset_in


class PaginationError(Exception):
    """
    Exception raised when item pagination fails.

    Attributes:
        message (str): Explanation of the error.
    """

    def __init__(self, message: str):
        super().__init__(message)


class QueryFormatError(Exception):
    """
    Exception raised when a query is improperly formatted.

    Attributes:
        message (str): Explanation of the error.
    """

    def __init__(self, message: str):
        super().__init__(message)



# This file is part of monday-client.
#
# Copyright (C) 2024 Leet Cyber Security <https://leetcybersecurity.com/>
#
# monday-client is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# monday-client is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with monday-client. If not, see <https://www.gnu.org/licenses/>.

# pylint: disable=redefined-outer-name

import json
from unittest.mock import AsyncMock, MagicMock

import pytest

from monday.client import MondayClient
from monday.services.boards import Boards


@pytest.fixture(scope="module")
def mock_client():
    return MagicMock(spec=MondayClient)


@pytest.fixture(scope="module")
def boards_instance(mock_client):
    return Boards(mock_client)


@pytest.mark.asyncio
async def test_query(boards_instance):
    mock_responses = [
        {'data': {'boards': [{'id': 1, 'name': 'Board 1'}, {'id': 2, 'name': 'Board 2'}]}},
        {'data': {'boards': [{'id': 3, 'name': 'Board 3'}]}},
        {'data': {'boards': []}}
    ]

    boards_instance.client.post_request = AsyncMock(side_effect=mock_responses)
    result = await boards_instance.query(board_ids=[1, 2, 3], boards_limit=2)

    assert result == [{'id': 1, 'name': 'Board 1'}, {'id': 2, 'name': 'Board 2'}, {'id': 3, 'name': 'Board 3'}]
    assert boards_instance.client.post_request.await_count == 3


@pytest.mark.asyncio
async def test_create(boards_instance):
    mock_response = {
        'data': {
            'create_board': {'id': 1, 'name': 'New Board'}
        }
    }

    boards_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await boards_instance.create(name="New Board")

    assert result == {'id': 1, 'name': 'New Board'}
    boards_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_duplicate(boards_instance):
    mock_response = {
        'data': {
            'duplicate_board': {'id': 2, 'name': 'Board 1 (copy)'}
        }
    }

    boards_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await boards_instance.duplicate(board_id=1)

    assert result == {'id': 2, 'name': 'Board 1 (copy)'}
    boards_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_update(boards_instance):
    mock_response = {
        'data': {
            'update_board': json.dumps({'id': 1, 'name': 'Updated Board'})
        }
    }

    boards_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await boards_instance.update(board_id=1, board_attribute="name", new_value="Updated Board")

    assert result == {'id': 1, 'name': 'Updated Board'}
    boards_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_archive(boards_instance):
    mock_response = {
        'data': {
            'archive_board': {'id': 1, 'state': 'archived'}
        }
    }

    boards_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await boards_instance.archive(board_id=1)

    assert result == {'id': 1, 'state': 'archived'}
    boards_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_delete(boards_instance):
    mock_response = {
        'data': {
            'delete_board': {'id': 1, 'state': 'deleted'}
        }
    }

    boards_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await boards_instance.delete(board_id=1)

    assert result == {'id': 1, 'state': 'deleted'}
    boards_instance.client.post_request.assert_awaited_once()


@pytest.mark.asyncio
async def test_get_groups(boards_instance):
    mock_response = {
        'data': {
            'boards': [
                {
                    'groups': [
                        {'id': 'group1', 'title': 'Group 1'},
                        {'id': 'group2', 'title': 'Group 2'}
                    ]
                }
            ]
        }
    }

    boards_instance.client.post_request = AsyncMock(return_value=mock_response)
    result = await boards_instance.get_groups(board_id=1)

    assert result == [{'id': 'group1', 'title': 'Group 1'}, {'id': 'group2', 'title': 'Group 2'}]
    boards_instance.client.post_request.assert_awaited_once()



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as read_me_file:
    long_description = read_me_file.read()

setup(
    name='monday-client',
    version='0.1.20',
    author='Dan Hollis',
    author_email='dh@leetsys.com',
    description='Python library for interacting with the monday.com API. Respects monday.com API rate limits and query complexity limits.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/LeetCyberSecurity/monday-client',
    packages=find_packages(include=['monday', 'monday.*']),
    install_requires=[
        'aiohttp',
        'pydantic',
        'requests'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.11',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3.11',
    license='GPLv3',
    project_urls={
        'Bug Reports': 'https://github.com/LeetCyberSecurity/monday-client/issues',
        'Source': 'https://github.com/LeetCyberSecurity/monday-client'
    },
)



# This file is part of monday-client.
#
# Copyright (C) 2024 Leet Cyber Security
#
# monday-client is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License.

# pylint: disable=redefined-outer-name

from unittest.mock import AsyncMock, patch

import aiohttp
import pytest

from monday.client import MondayClient


@pytest.fixture(scope="module")
def client_instance():
    client = MondayClient("test_api_key")
    client.max_retries = 2
    client.rate_limit_seconds = 1
    return client


@pytest.mark.asyncio
async def test_init():
    client = MondayClient("test_api_key")
    assert client.url == 'https://api.monday.com/v2'
    assert client.headers == {
        'Content-Type': 'application/json',
        'Authorization': 'test_api_key'
    }
    assert client.rate_limit_seconds == 60
    assert client.max_retries == 4


@pytest.mark.asyncio
async def test_post_request_success(client_instance):
    mock_response = {'data': {'some': 'data'}}

    with patch.object(client_instance, '_execute_request', new_callable=AsyncMock, return_value=mock_response):
        result = await client_instance.post_request("test_query")

    assert result == mock_response


@pytest.mark.asyncio
async def test_post_request_complexity_limit_exceeded(client_instance):
    error_responses = [
        {'error': 'error', 'error_code': 'ComplexityException', 'error_message': 'Complexity limit exceeded reset in 0.1 seconds'},
        {'data': {'some': 'data'}}
    ]

    client_instance.max_retries = 2

    with patch.object(client_instance, '_execute_request', side_effect=error_responses):
        with patch('monday.client.asyncio.sleep', new_callable=AsyncMock) as mock_sleep:
            result = await client_instance.post_request("test_query")

    mock_sleep.assert_called_once_with(1)
    assert result == {'data': {'some': 'data'}}


@pytest.mark.asyncio
async def test_post_request_mutation_limit_exceeded(client_instance):
    error_responses = [
        {'error': 'error', 'status_code': 429},
        {'data': {'some': 'data'}}
    ]

    with patch.object(client_instance, '_execute_request', side_effect=error_responses):
        with patch('monday.client.asyncio.sleep', new_callable=AsyncMock) as mock_sleep:
            result = await client_instance.post_request("test_query")

    mock_sleep.assert_called_once_with(1)
    assert result == {'data': {'some': 'data'}}


@pytest.mark.asyncio
async def test_post_request_max_retries_reached(client_instance):
    client_instance.max_retries = 1
    error_responses = [
        {'error': 'error', 'error_code': 'ComplexityException', 'error_message': 'Complexity limit exceeded reset in 0.1 seconds'},
        {'data': {'some': 'data'}}
    ]

    with patch.object(client_instance, '_execute_request', side_effect=error_responses):
        with patch('monday.client.asyncio.sleep', new_callable=AsyncMock):
            result = await client_instance.post_request("test_query")

    assert 'Max retries reached' in result['error']


@pytest.mark.asyncio
async def test_post_request_client_error_retry(client_instance):
    error_responses = [
        aiohttp.ClientError('Client error occurred'),
        aiohttp.ClientError('Client error occurred'),
        {'data': {'some': 'data'}}
    ]
    client_instance.max_retries = 3

    with patch.object(client_instance, '_execute_request', side_effect=error_responses):
        with patch('monday.client.asyncio.sleep', new_callable=AsyncMock) as mock_sleep:
            result = await client_instance.post_request("test_query")

    assert mock_sleep.call_count == 2
    mock_sleep.assert_called_with(60)
    assert result == {'data': {'some': 'data'}}


@pytest.mark.asyncio
async def test_post_request_max_retries_client_error(client_instance):
    client_error = aiohttp.ClientError('Client error occurred')

    with patch.object(client_instance, '_execute_request', side_effect=client_error):
        with patch('monday.client.asyncio.sleep', new_callable=AsyncMock):
            result = await client_instance.post_request("test_query")

    assert 'Max retries reached' in result['error']


@pytest.mark.asyncio
async def test_execute_request(client_instance):
    mock_response = AsyncMock()
    mock_json = AsyncMock(return_value={'data': {'some': 'data'}})
    mock_response.__aenter__.return_value.json = mock_json

    with patch('aiohttp.ClientSession.post', return_value=mock_response) as mock_post:
        # pylint: disable=W0212
        result = await client_instance._execute_request("test_query")
        # pylint: enable=W0212

    assert result == {'data': {'some': 'data'}}
    mock_post.assert_called_once_with(
        'https://api.monday.com/v2',
        json={'query': 'test_query'},
        headers={'Content-Type': 'application/json', 'Authorization': 'test_api_key'}
    )
